@@ -1,23 +1,36 @@
-from typing import Any, List, Type
+from typing import Any, Type
 import builtins
 import json
 import copy
+import sys
 
 
 class ArgumentParser:
     """A simple argument parser.
     It allows you to specify the type of each expected argument.
 
-    The key-shorts and their values have to be given together.
+        The key-shorts and their values have to be given together.
     For example: if an integer \"offset\" with short being \"o\" has to take the value 100, in the command-line you would type \"-o100\".
     """
 
-    def __init__(self):
+    def __init__(self, description: str = "", epilog: str = "", add_help: bool = True):
         """
         Initialisation of the class.
+
+        Parameters
+        ----------
+        description: string
+            description of the program purpose (default is "")
+        epilog: string
+            last text displayed by help (default is "")
+        add_help: bool
+            flag to allow help to be provided (default is True)
         """
         self.arguments: dict = {}
         self.__results: dict = {}
+        self.description = description
+        self.epilog = epilog
+        self.add_help = add_help
 
     def from_json(self, path: str):
         """Use the content of the json file at path to fill list of arguments to parse.
@@ -32,11 +45,12 @@ class ArgumentParser:
 
         for key, value in data.items():
             short = value["short"]
-            type = getattr(builtins, value["type"])
+            type = getattr(builtins, value["type"]) if "type" in value.keys() else str
             required = value["required"] if "required" in value.keys() else True
             description = value["description"] if "description" in value.keys() else ""
+            choices = value["choices"] if "choices" in value.keys() else []
 
-            self.add(key, short, type, required, description)
+            self.add(key, short, type, required, description, choices)
 
     def add(
         self,
@@ -45,6 +59,7 @@ class ArgumentParser:
         type: type = Type[str],
         required: bool = True,
         description: str = "",
+        choices: list = [],
     ) -> None:
         """Add an argument to parse.
 
@@ -58,6 +73,10 @@ class ArgumentParser:
             type of the argument. (default is str)
         required: boolean
             flag that specify if a key is mandatory. (default is True)
+        description: str
+            description of the argument. (default is "")
+        choices: list
+            allowed values for the argument. (default is [])
         """
         assert key not in self.arguments.keys(), "Key already used."
         assert isinstance(key, str), "Key must be a string."
@@ -65,6 +84,7 @@ class ArgumentParser:
         assert isinstance(type, Type), "Type must be a class type."
         assert isinstance(required, bool), "Required must be a boolean."
         assert isinstance(description, str), "Description must be a string."
+        assert isinstance(choices, list), "Choices must be a list."
         assert len(short) == 1, "Short must be a single character."
 
         self.arguments[key] = {
@@ -72,9 +92,10 @@ class ArgumentParser:
             "required": required,
             "type": type,
             "description": description,
+            "choices": [str(c) for c in choices],
         }
 
-    def compile(self, args: List[str]):
+    def compile(self, args: list[str]):
         """Parse the input arguments with the keys previously specified.
 
         Parameters
@@ -83,11 +104,12 @@ class ArgumentParser:
             Arguments to parse
 
         """
-        if len(args) == 0 and len(self.arguments) != 0:
+
+        if len(args) == 0 and len(self.arguments) != 0 and self.add_help:
             print("To get help, use -h or --help command line options.")
             exit()
 
-        if len(args) == 1 and args[0] in ["--help", "-h"]:
+        if len(args) == 1 and args[0] in ["--help", "-h"] and self.add_help:
             self.show_help()
             exit()
 
@@ -97,7 +119,15 @@ class ArgumentParser:
         for arg in args:
             if arg[:2] in shorts:
                 key = keys[shorts.index(arg[:2])]
-                self.__results[key] = arg[2:]
+                choices = self.arguments[key]["choices"]
+                if choices:
+                    if arg[2:] in choices:
+                        self.__results[key] = arg[2:]
+                    else:
+                        print(f"Provided {key} not in {choices}")
+                        exit()
+                else:
+                    self.__results[key] = arg[2:]
 
         for key in keys:
             if self.arguments[key]["required"] and key not in self.__results.keys():
@@ -106,31 +136,50 @@ class ArgumentParser:
                 )
                 exit()
 
+        return {key: self.__getitem__(key) for key in self.arguments}
+
     def show_help(self):
         """Displays help for the argument to pass on the command-line."""
         n_arg = len(self.arguments)
 
         if n_arg != 0:
             length_str = max([len(arg) for arg in self.arguments.keys()])
+            script_name = sys.argv[0]
 
-            print(f"{n_arg} argument{'s' if n_arg>1 else ''} expected: ")
+            arg_strs: list[str] = []
             for key, value in self.arguments.items():
-                req_str = "required" if value["required"] else ""
-                print(
-                    f"{value['short']} {value['type']}: {key:{length_str+10}s}{req_str}"
-                )
-                print(f"\t{value['description']}")
+                if value["required"]:
+                    arg_strs.append(f"{value['short']}{key}")
 
-            print("\nUsage:")
-            print("Short keys and their values must be together. For example:")
-            print("- '-o12' for a short key being 'o' and value '12'.")
-            print("- '-dpath' for a short key being 'd' and value 'path'.")
-            print(
-                "\nNon-necessary arguments with no values provided will take the default constructor value."
-            )
-        else:
-            print("Usage:")
-            print("No argument expected. Just run your script as you would do normaly.")
+            for key, value in self.arguments.items():
+                if not value["required"]:
+                    arg_strs.append(f"[{value['short']}{key}]")
+
+            print(f"usage: {script_name} {' '.join(arg_strs)}\n")
+            if self.description:
+                print(f"{self.description}\n")
+
+            print(f"positional arguments:")
+            for key, value in self.arguments.items():
+                if value["required"]:
+                    print(f"{value['short']}: {key:{length_str+10}s} {value['type']}")
+                    print(f"\t{value['description']}")
+                    if value["choices"]:
+                        print(f"\tPossible values are {value['choices']}.")
+
+            print("")
+            print(f"optional arguments:")
+            for key, value in self.arguments.items():
+                if not value["required"]:
+                    print(f"{value['short']}: {key:{length_str+10}s} {value['type']}")
+                    print(f"\t{value['description']}")
+                    if value["choices"]:
+                        print(f"\tPossible values are {value['choices']}")
+
+            print("-h, --help\n\tshow this help message and exit")
+
+            if self.epilog:
+                print(f"\n{self.epilog}")
 
     def to_json(self, filename: str):
         """Export the arguments dictionnary to a json file.
@@ -165,13 +214,17 @@ class ArgumentParser:
         """
         assert key in self.arguments, f'Key "{key}" not found.'
 
+        value_type = self.arguments[key]["type"]
+
         if key not in self.__results:
-            return self.arguments[key]["type"]()
+            return value_type()
 
         try:
-            return self.arguments[key]["type"](self.__results[key])
+            if value_type == bool:
+                return value_type(eval(self.__results[key]))
+            return value_type(self.__results[key])
         except ValueError as e:
             print(
                 f"{__class__.__name__} error with '{key}': {e}. Using default constructor value."
             )
-            return self.arguments[key]["type"]()
+            return value_type()
